import asyncio
import json
import websockets


class Handler:
    def __init__(self, url) -> None:
        self.url = url
        self.ws = None
        self.loop = asyncio.get_event_loop()
        # perform a synchronous connect
        self.loop.run_until_complete(self.__async__connect())

    def event(self, inputs):
        return self.loop.run_until_complete(self.__async__send_return(inputs))

    def send(self, file):
        return self.loop.run_until_complete(self.__async__send(file))

    def receive(self):
        return self.loop.run_until_complete(self.__async__receive())

    def close(self):
        return self.loop.run_until_complete(self.__async__close())

    async def __async__close(self):
        await self.ws.close()

    async def __async__connect(self):
        # print("Attempting to connect at {}".format(self.url))
        self.ws = await websockets.connect(self.url)
        # print("Connected")

    async def __async__send(self, inputs):
        await self.ws.send(inputs)

    async def __async__send_return(self, inputs):
        await self.ws.send(inputs)
        data = await self.ws.recv()
        message = json.loads(data)
        return message["content"]

    async def __async__receive(self):
        data = await self.ws.recv()
        return data
