import os
from handler import Handler
import codecs
import argparse
import json
import urllib.request
import webbrowser
import _version

class CommandLineParser:
    def __init__(self):
        # Build the command line parser
        self.ap = argparse.ArgumentParser(usage=self.usage())

        # Add the arguments to the command line parser
        self.ap.add_argument("-t", "--test", action='store_false', default=True,
                             help=argparse.SUPPRESS)
        self.ap.add_argument('-m2', action='store_true', default=False,
                             help=argparse.SUPPRESS)

        self.required = self.ap.add_argument_group('required arguments')
        self.required.add_argument("-v", "--verilog", nargs='+', required=True,
                                   help="Verilog file in the Pitanga supported language set")
        self.required.add_argument("-p", "--pinout",
                                   help="Pinout file in the Pitanga format syntax")

    def get_arguments(self):
        return vars(self.ap.parse_args())
    
    def usage(self):
        return '''pitanga.py -v VERILOG -p PINOUT'''


if __name__ == '__main__':
    print(f'{_version.copyright_header}')

    # Construct the argument parser
    cl_parser = CommandLineParser()
    args = cl_parser.get_arguments()

    # If set, the argument passed in command line will not be parsed and used by Kivy.
    os.environ["KIVY_NO_ARGS"] = "1"
    # If set, logs will be not print to the console
    os.environ["KIVY_NO_CONSOLELOG"] = "1"

import kivy
from kivy.config import Config
Config.set('input', 'mouse', 'mouse,multitouch_on_demand')
from kivy.properties import ObjectProperty, StringProperty
from kivy.uix.widget import Widget
from kivy.uix.image import AsyncImage, Image
from kivy.app import App
from kivy.lang import Builder
from kivy.clock import Clock

kivy.require('1.9.1')

Config.set('graphics', 'width', '850')
Config.set('graphics', 'height', '550')

class PitangaBoard(Widget):
    sw0 = ObjectProperty(None)
    sw1 = ObjectProperty(None)
    sw2 = ObjectProperty(None)
    sw3 = ObjectProperty(None)
    sw4 = ObjectProperty(None)
    sw5 = ObjectProperty(None)
    sw6 = ObjectProperty(None)
    sw7 = ObjectProperty(None)
    sw8 = ObjectProperty(None)
    sw9 = ObjectProperty(None)
    btn0 = ObjectProperty(None)
    btn1 = ObjectProperty(None)
    btn2 = ObjectProperty(None)
    led0 = ObjectProperty(None)
    led1 = ObjectProperty(None)
    led2 = ObjectProperty(None)
    led3 = ObjectProperty(None)
    led4 = ObjectProperty(None)
    led5 = ObjectProperty(None)
    led6 = ObjectProperty(None)
    led7 = ObjectProperty(None)
    led8 = ObjectProperty(None)
    led9 = ObjectProperty(None)
    segd0 = ObjectProperty(None)
    segd1 = ObjectProperty(None)
    segd2 = ObjectProperty(None)
    segd3 = ObjectProperty(None)
    banners = ObjectProperty(None)

    def __init__(self, websocket):
        super().__init__()
        # link outside chip component values to Pitanga chip port
        self.banners.build()
        self.INPUTS = 18
        self.OUTPUTS = 42
        self.p = [False, True, self.sw0.active, self.sw1.active, self.sw2.active, self.sw3.active, self.sw4.active,
                  self.sw5.active, self.sw6.active, self.sw7.active, self.sw8.active, self.sw9.active,
                  False, False, False, False, False, False, self.led0.on, self.led1.on, self.led2.on, self.led3.on,
                  self.led4.on, self.led5.on, self.led6.on, self.led7.on, self.led8.on, self.led9.on,
                  self.segd0.a_on, self.segd0.b_on, self.segd0.c_on, self.segd0.d_on, self.segd0.e_on,
                  self.segd0.f_on, self.segd0.g_on, self.segd0.dp_on, self.segd1.a_on, self.segd1.b_on,
                  self.segd1.c_on, self.segd1.d_on, self.segd1.e_on, self.segd1.f_on, self.segd1.g_on,
                  self.segd1.dp_on, self.segd2.a_on, self.segd2.b_on, self.segd2.c_on, self.segd2.d_on,
                  self.segd2.e_on, self.segd2.f_on, self.segd2.g_on, self.segd2.dp_on, self.segd3.a_on,
                  self.segd3.b_on, self.segd3.c_on, self.segd3.d_on, self.segd3.e_on, self.segd3.f_on,
                  self.segd3.g_on, self.segd3.dp_on]
        self.handler = websocket
        inputs = self.get_inputs()
        self.next_state = self.handler.event(json.dumps({'type':'signal','content':inputs}))
        self.set_outputs()
        self.update()

    def set_outputs(self, port=-1):
        outputs = []
        outputs[:0] = self.next_state
        for x in range(self.INPUTS, self.INPUTS + self.OUTPUTS):
            self.p[x] = True if outputs[(self.INPUTS+1)*x + (port+1)] == '1' else False

    def get_inputs(self):
        ports = ''
        for x in range(self.INPUTS):
            ports += '1' if self.p[x] == True else '0'
        return ports

    def event(self, switchobject, state, port):
        self.p[port] = state
        self.set_outputs(port)
        self.update()
        inputs = self.get_inputs()
        self.next_state = self.handler.event(json.dumps({'type':'signal','content':inputs}))

    def update(self):
        # Update leds connected to ports
        self.led0.on = self.p[18]
        self.led1.on = self.p[19]
        self.led2.on = self.p[20]
        self.led3.on = self.p[21]
        self.led4.on = self.p[22]
        self.led5.on = self.p[23]
        self.led6.on = self.p[24]
        self.led7.on = self.p[25]
        self.led8.on = self.p[26]
        self.led9.on = self.p[27]
        self.segd0.a_on = self.p[28]
        self.segd0.b_on = self.p[29]
        self.segd0.c_on = self.p[30]
        self.segd0.d_on = self.p[31]
        self.segd0.e_on = self.p[32]
        self.segd0.f_on = self.p[33]
        self.segd0.g_on = self.p[34]
        self.segd0.dp_on = self.p[35]
        self.segd1.a_on = self.p[36]
        self.segd1.b_on = self.p[37]
        self.segd1.c_on = self.p[38]
        self.segd1.d_on = self.p[39]
        self.segd1.e_on = self.p[40]
        self.segd1.f_on = self.p[41]
        self.segd1.g_on = self.p[42]
        self.segd1.dp_on = self.p[43]
        self.segd2.a_on = self.p[44]
        self.segd2.b_on = self.p[45]
        self.segd2.c_on = self.p[46]
        self.segd2.d_on = self.p[47]
        self.segd2.e_on = self.p[48]
        self.segd2.f_on = self.p[49]
        self.segd2.g_on = self.p[50]
        self.segd2.dp_on = self.p[51]
        self.segd3.a_on = self.p[52]
        self.segd3.b_on = self.p[53]
        self.segd3.c_on = self.p[54]
        self.segd3.d_on = self.p[55]
        self.segd3.e_on = self.p[56]
        self.segd3.f_on = self.p[57]
        self.segd3.g_on = self.p[58]
        self.segd3.dp_on = self.p[59]

    def clock_1Hz(self, dt):
        clock_port = 15
        square_wave = not self.p[clock_port]
        self.event('swobj', square_wave, clock_port)

    def clock0_5Hz(self, dt):
        clock_port = 16
        square_wave = not self.p[clock_port]
        self.event('swobj', square_wave, clock_port)

    def clock0_1Hz(self, dt):
        clock_port = 17
        square_wave = not self.p[clock_port]
        self.event('swobj', square_wave, clock_port)

    def next_banner(self, dt):
        self.banners.carousel.load_next()

class MyImage(AsyncImage):
    source = StringProperty(None)
    url_to_open = StringProperty(None)
    

class Banner(Widget):
    def build(self):

        try: 
            mapfile = urllib.request.urlopen("https://www.inplace-da.com.br/ads.txt")
            mapfile = mapfile.read().decode(encoding='utf-8').split('\n')
            for line in mapfile:
                line = line.split(',')
                src = line[0]
                url = line[1]
                image = MyImage(source=src, url_to_open=url)
                self.carousel.add_widget(image)
        except:
            image = MyImage(source="images/banner-inplace.png", url_to_open="https://apoia.se/pitanga-community")
            self.carousel.add_widget(image)
            image = MyImage(source="images/banner-ufrgs.png", url_to_open="https://www.inf.ufrgs.br/cei/")
            self.carousel.add_widget(image)

    def on_touch_up(self, touch):
        if self.collide_point(touch.x, touch.y):
            self.__open_url(self.carousel.current_slide.url_to_open)

    def __open_url(self, url):
        # print(f"Open URL: {url}")
        webbrowser.open(url)


class PitangaApp(App):
    def __init__(self, websocket, top_level_name):
        super().__init__()
        self.websocket = websocket
        self.top_level_name = top_level_name

    def build(self):
        Builder.load_file('gui/pitanga.kv')
        self.title = f'Pitanga Community Edition - {self.top_level_name}'
        board = PitangaBoard(self.websocket)
        Clock.schedule_interval(board.clock_1Hz, 1/2)
        Clock.schedule_interval(board.clock0_5Hz, 2/2)
        Clock.schedule_interval(board.clock0_1Hz, 10/2)
        Clock.schedule_interval(board.next_banner, 10)
        return board


def read_file_pinout(filename):
    # Read the file
    try:
        file = open(filename, "rb")
    except FileNotFoundError:
        print(f'ERROR: The input file <{filename}> does not exist.')
        print('Exiting...')
        exit()
    except TypeError:
        print(f'ERROR: A pinout file is required.')

    # Check if file is not empty
    if os.stat(filename).st_size == 0:
        print(f'ERROR: The input file <{filename}> is empty.')
        print('Exiting...')
        exit()

    read_file = codecs.decode(file.read(), 'utf-8', 'strict')
    file.close()
    return read_file

def read_file_verilog(filename_list: list):
    # Read the file
    read_file = []
    for filename in filename_list:
        try:
            file = open(filename, "rb")
        except FileNotFoundError:
            print(f'ERROR: The input file <{filename}> does not exist.')
            print('Exiting...')
            exit()
        except TypeError:
            print(f'ERROR: A pinout file is required.')

        # Check if file is not empty
        if os.stat(filename).st_size == 0:
            print(f'ERROR: The input file <{filename}> is empty.')
            print('Exiting...')
            exit()

        read_file.append(codecs.decode(file.read(), 'utf-8', 'strict'))
        file.close()
    return read_file


def setup_server_connection(production_server):
    if production_server:
        # connect to the production server
        websocket_var = Handler('ws://34.95.183.119:9090')
    else:
        # connect to a local server
        websocket_var = Handler('ws://localhost:9090')
    return websocket_var


if __name__ == '__main__':
    if args['m2']:
        filename = args['verilog']
        verilog_description = read_file_verilog(filename)
        server_websocket = setup_server_connection(args['test'])
        message = json.dumps({'type':'verilog', 'filename': filename, 'content':verilog_description})
        server_websocket.send(message)
        message = json.dumps({'type':'m2','content':''})
        server_websocket.send(message)
        message = server_websocket.receive()
        error_msg = json.loads(message)
        if error_msg["type"] == 'exception' and error_msg["content"] != 'No Err':
            print(error_msg["content"])
            print('Exiting...')
            server_websocket.close()
            exit()
        else:
            verilog = json.loads(server_websocket.receive())
            pinout = json.loads(server_websocket.receive())
            file = os.path.splitext(filename[0])[0]
            to_write = open(f'{file}_out.v', 'w')
            to_write.write(verilog['content'])
            to_write.close()
            to_write = open(f'{file}_out.pinout', 'w')
            to_write.write(pinout['content'])
            to_write.close()
            print('Successful Translation...')
        server_websocket.close()
    else:
        # Read pinout and verilog files;
        pinout_scheme = read_file_pinout(args['pinout'])
        verilog_description = read_file_verilog(args['verilog'])

        # Setup connection to server (Dev, DevOps or Cloud server)
        server_websocket = setup_server_connection(args['test'])
        
        verilog_filenames = []
        for file in args['verilog']:
            verilog_filenames.append(os.path.basename(file))
        pinout_filename = os.path.basename(args['pinout'])

        # Send pinout and verilog files to the server;
        message = json.dumps({'type':'verilog','filename': verilog_filenames, 'content':verilog_description})
        server_websocket.send(message)
        message = json.dumps({'type':'pinout','filename': pinout_filename, 'content':pinout_scheme})
        server_websocket.send(message)

        message = server_websocket.receive()
        error_msg = json.loads(message)
        if error_msg["type"] == 'exception' and error_msg["content"] != 'No Err':
            print(error_msg["content"])
            print('Exiting...')
            server_websocket.close()
            exit()
        else:
            print('INFO: Verilog compilation successful.')
            print('INFO: Pinout compilation successful.')

            message = server_websocket.receive()
            message = json.loads(message)
            if message["type"] == 'report':
                print(message["content"])

        # Run the board
        verilog_toplevel_name = os.path.splitext(os.path.basename(args['pinout']))[0]
        board = PitangaApp(server_websocket, verilog_toplevel_name)
        board.run()

        # Close connection to server
        server_websocket.close()
