module rotate10b(
    input   clk,        // 1Hz system clock (see pinout file for details)
    input   rst,        // System reset: set all flip-flops to '0'
    input   set,        // System set  : set all flip-flops to '1'

    input   mode,       // Invert the most significant bit (see pinout for details)
    input   cen,        // Clock enable is connected to VCC (see pinout for details)
    output  [9:0] q);   // 10-bit rotate register output

    // Connect the flip-flops serially to build the rotate register
    wire    [9:0] q_sig;

    // Invert the most significant bit and close the loop to the less significant bit
    wire    xnor2q0;
    xnor(xnor2q0,q_sig[9],mode);

    // Rotate register connections
    DFFRSE dff0(.q(q_sig[0]), .d(xnor2q0) , .clk(clk), .reset(rst), .set(set), .enable(cen));
    DFFRSE dff1(.q(q_sig[1]), .d(q_sig[0]), .clk(clk), .reset(rst), .set(set), .enable(cen));
    DFFRSE dff2(.q(q_sig[2]), .d(q_sig[1]), .clk(clk), .reset(rst), .set(set), .enable(cen));
    DFFRSE dff3(.q(q_sig[3]), .d(q_sig[2]), .clk(clk), .reset(rst), .set(set), .enable(cen));
    DFFRSE dff4(.q(q_sig[4]), .d(q_sig[3]), .clk(clk), .reset(rst), .set(set), .enable(cen));
    DFFRSE dff5(.q(q_sig[5]), .d(q_sig[4]), .clk(clk), .reset(rst), .set(set), .enable(cen));
    DFFRSE dff6(.q(q_sig[6]), .d(q_sig[5]), .clk(clk), .reset(rst), .set(set), .enable(cen));
    DFFRSE dff7(.q(q_sig[7]), .d(q_sig[6]), .clk(clk), .reset(rst), .set(set), .enable(cen));
    DFFRSE dff8(.q(q_sig[8]), .d(q_sig[7]), .clk(clk), .reset(rst), .set(set), .enable(cen));
    DFFRSE dff9(.q(q_sig[9]), .d(q_sig[8]), .clk(clk), .reset(rst), .set(set), .enable(cen));

    // Rotate register connections to the board (leds)
    buf(q[0], q_sig[0]);
    buf(q[1], q_sig[1]);
    buf(q[2], q_sig[2]);
    buf(q[3], q_sig[3]);
    buf(q[4], q_sig[4]);
    buf(q[5], q_sig[5]);
    buf(q[6], q_sig[6]);
    buf(q[7], q_sig[7]);
    buf(q[8], q_sig[8]);
    buf(q[9], q_sig[9]);
endmodule
